"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.calculatePricingInformation = void 0;
const geo_shipping_prices_1 = require("@uppfirst/geo-shipping-prices");
const currencyCodeToSymbolMap = {
    'usd': '$',
    'eur': '€',
    'gbp': '£',
    'jpy': '¥',
};
const convertToFloat = (value, decimals) => {
    return parseFloat((value / Math.pow(10, decimals)).toFixed(decimals));
};
const calculateShippingPrices = ({ product, stateCode, countryCode, }) => {
    var _a, _b;
    const shipping = product.shipping;
    if (!shipping || !countryCode) {
        return {
            taxCost: 0,
            taxPercent: 0,
            shippingCost: 0,
        };
    }
    const shippingCalculator = new geo_shipping_prices_1.Shipping(shipping);
    const prices = stateCode && countryCode === 'US'
        ? shippingCalculator.state(stateCode)
        : shippingCalculator.country(countryCode);
    const taxPercent = ((_a = prices.prices) === null || _a === void 0 ? void 0 : _a.tax) || 0;
    const shippingCost = ((_b = prices.prices) === null || _b === void 0 ? void 0 : _b.price) || 0;
    return {
        taxCost: (taxPercent / 100) * product.price,
        taxPercent,
        shippingCost: shippingCost * 100,
    };
};
const calculatePricingInformation = ({ product, countryCode, stateCode, combination, usedRewards, }) => {
    const startAmountPercent = product.startAmountPercent / 100;
    const rewardShare = product.rewardShare / 100;
    const claimPercent = 1 - startAmountPercent;
    const traitCost = ((combination === null || combination === void 0 ? void 0 : combination.price) || 0) * 100;
    const baseReserve = product.price * startAmountPercent;
    const baseClaim = product.price * claimPercent;
    const gainedRewardsReserve = rewardShare * baseReserve;
    const gainedRewardsClaim = rewardShare * baseClaim;
    const totalBase = product.price;
    const { shippingCost, taxCost, taxPercent } = calculateShippingPrices({
        product,
        countryCode,
        stateCode,
    });
    const usedRewardsTotal = usedRewards.reduce((acc, reward) => acc + reward.worth, 0);
    return {
        currency: {
            code: product.currency.symbol.toLowerCase(), // Lowercase currency code (usd, eur, etc.)
            symbol: currencyCodeToSymbolMap[product.currency.symbol.toLocaleLowerCase()],
            decimals: product.currency.decimals
        },
        startAmountPercent: product.startAmountPercent,
        usedRewardsTotal: convertToFloat(usedRewardsTotal, product.currency.decimals),
        reserve: {
            base: convertToFloat(baseReserve, product.currency.decimals),
            traitCost: convertToFloat(traitCost, product.currency.decimals),
            gainedRewardsAmount: convertToFloat(gainedRewardsReserve, product.currency.decimals),
            taxPercent,
            shippingCost: convertToFloat(shippingCost, product.currency.decimals),
            taxCost: convertToFloat(taxCost, product.currency.decimals),
            total: convertToFloat(Math.max(baseReserve + traitCost + taxCost + shippingCost - usedRewardsTotal, 0), product.currency.decimals),
        },
        claim: {
            base: convertToFloat(baseClaim, product.currency.decimals),
            gainedRewardsAmount: convertToFloat(gainedRewardsClaim, product.currency.decimals),
            total: convertToFloat(Math.max(baseClaim - usedRewardsTotal), product.currency.decimals),
        },
        total: {
            base: convertToFloat(totalBase, product.currency.decimals),
            gainedRewardsAmount: convertToFloat(gainedRewardsReserve + gainedRewardsClaim, product.currency.decimals),
            total: convertToFloat(Math.max(totalBase + taxCost + shippingCost - usedRewardsTotal, 0), product.currency.decimals),
        },
    };
};
exports.calculatePricingInformation = calculatePricingInformation;
